package com.gic.demo.single;

import com.gic.demo.single.service.DemoStoreService;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;
import org.springframework.stereotype.Component;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.function.Consumer;

/**
 * @author zhurz
 */
@Component
public class TestRun {

    @Value("${jdbc.maxsize}")
    private Integer jdbcMaxsize;

    public void test(Consumer<String> consumer) throws InterruptedException {
        int maxThreadSize = 100;
        int size = 100000;
        ExecutorService exec = Executors.newFixedThreadPool(maxThreadSize);
        CountDownLatch countDownLatch = new CountDownLatch(size);
        long begin = System.currentTimeMillis();
        for (int i = 0; i < size; i++) {
            int finalI = i;
            exec.execute(() -> {
                try {
                    consumer.accept("sss" + finalI);
                } finally {
                    countDownLatch.countDown();
                }
            });
        }
        countDownLatch.await();
        long now = System.currentTimeMillis();
        double cost = ((now - begin) / 1000d);
        System.out.printf(
                "并发线程: %d, jdbc最大连接: %d, 查询次数: %d, 耗时:%.2f s, QPS: %.2f\n",
                maxThreadSize,
                jdbcMaxsize,
                size,
                cost,
                size / cost
        );
    }

    public static void main(String[] args) throws InterruptedException {
        ApplicationContext applicationContext = new ClassPathXmlApplicationContext("applicationContext-conf.xml");
        TestRun bean = applicationContext.getBean(TestRun.class);
        DemoStoreService demoStoreService = applicationContext.getBean(DemoStoreService.class);
        System.out.println("无事务");
        bean.test(demoStoreService::test1);
        System.out.println("开启事务");
        bean.test(demoStoreService::test2);
        System.out.println("只读事务");
        bean.test(demoStoreService::test3);
    }
}
